EDDI = "eddi"
ZAPPI = "zappi"
HARVI = "harvi"
LIBBI = "libbi"
DEVICE_TYPES = [EDDI, ZAPPI, HARVI, LIBBI]
HOUR = "hour"
MINUTE = "minute"
CT_GENERATION = "Generation"
CT_BATTERY = "AC Battery"
CT_LOAD = "Internal Load"
CT_MONITOR = "Monitor"
CT_GRID = "Grid"
FREQUENCY_GRID = "frequency"
VOLTAGE_GRID = "voltage"
